/**
 * @file ncogs.c
 * This is the main program start point.
 * The original version of this program was posted by Rsadeika as RBot1.c .
 *
 * There is nothing novel about this code. Use at your own risk.
 *
 * It has been expanded into a multi-cog test program.
 *
 * The way it works:
 * The "count" number of cogs will be started.
 * On each main program pass, the LED will blink on for 1 second.
 * Then "count" number of short blinks should happen.
 *
 * The LED to blink is saved in the "pin" variable. It is set to 0.
 */
#include <propeller.h>
#include <stdio.h>
#include <stdlib.h>

int pin   = 0;  // set to your LED pin number
int count = 2;  // set to the number of cogs (short blinks per pass).

void do_job1(void *arg)
{
    volatile int *start = (int*)arg;

    while(1) {
        while(*start == 0)
            ; // do nothing until start
        high(pin);
        waitcnt(CLKFREQ/20 + CNT);
        low(pin);
        *start = 0;
    }
}

int high(int WCpin)
{
    unsigned int bits = 1 << WCpin;
    DIRA |= bits;
    OUTA |= bits;
    return 0;
}

int low(int WCpin)
{
    unsigned int mask = 1 << WCpin;
    DIRA |= mask;
    OUTA &= ~mask;
    return 0;
}

/**
 * Main program function.
 */
int main(void)
{

    int stacksize = sizeof(_thread_state_t)+sizeof(int)*5;
    int *stack[8];
    int slot[8];

    int n;
    int pass = 0;
    
    waitcnt(CLKFREQ/2 + CNT);
    for(n = 0; n < count; n++) {
        stack[n] = (int*) malloc(stacksize);
        if(stack[n] == 0) {
            printf("Malloc failed\n");
            return;
        }
        slot[n] = 0;
        printf("Start COG %d\n", n+1);
        cogstart(do_job1, (void*)&slot[n], &stack[n], stacksize); 
        waitcnt(CLKFREQ/10 + CNT);
    }
                
    while(1)
    {
        printf("Pass count %d\n", ++pass);
        high(pin);
        waitcnt(CLKFREQ + CNT); 
        low(pin);
        waitcnt(CLKFREQ/2 + CNT); 
        for(n = 0; n < count; n++) {
            slot[n] = -1;
            waitcnt(CLKFREQ/2 + CNT);
        }
    }
    
    // just a reminder. never called.
    // not necessary in this context.
    for(n = 0; n < count; n++) {
        free(stack[n]);
    }
    return 0;
}
